"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var utils_1 = require("./utils");
var constants_1 = require("./constants");
var Touch = (function (_super) {
    __extends(Touch, _super);
    function Touch() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.node = null;
        _this.lastX = null;
        _this.lastY = null;
        _this.panStart = null;
        _this.panned = false;
        _this.lastDistance = 0;
        _this.zoomLevel = 0;
        _this._setup = function (node) {
            if (!node)
                return;
            _this.node = node;
            _this.node.style.display = 'block';
            _this.node.style.margin = '0px';
            _this.node.style.padding = '0px';
        };
        _this._reset = function () {
            var onReset = _this.props.onReset;
            utils_1.reset();
            _this.zoomLevel = 0;
            if (onReset)
                onReset();
        };
        _this._updateCoordinatesMouse = function (event) {
            _this.lastX = event.pageX - _this.node.offsetLeft;
            _this.lastY = event.pageY - _this.node.offsetTop;
        };
        _this._updateCoordinatesTouch = function (event) {
            _this.lastX = event.targetTouches[0].pageX - _this.node.offsetLeft;
            _this.lastY = event.targetTouches[0].pageY - _this.node.offsetTop;
        };
        _this._onMouseDown = function (event) {
            event.preventDefault();
            _this._updateCoordinatesMouse(event);
            return _this._onPanStart(event);
        };
        _this._onTouchStart = function (event) {
            _this._updateCoordinatesTouch(event);
            if (event.targetTouches.length > 1)
                return _this._onPinchStart(event);
            return _this._onPanStart(event);
        };
        _this._onPanStart = function (event) {
            var onPanStart = _this.props.onPanStart;
            _this.panStart = utils_1.transformedPoint(_this.lastX, _this.lastY);
            _this.panned = false;
            if (onPanStart)
                onPanStart(event);
        };
        _this._onMouseMove = function (event) {
            event.preventDefault();
            _this._updateCoordinatesMouse(event);
            return _this._onPanMove(event);
        };
        _this._onTouchMove = function (event) {
            event.preventDefault();
            _this._updateCoordinatesTouch(event);
            if (event.targetTouches.length > 1)
                return _this._onPinch(event);
            return _this._onPanMove(event);
        };
        _this._onPanMove = function (event) {
            var _a = _this.props, onPan = _a.onPan, onTranslate = _a.onTranslate;
            _this.panned = true;
            if (_this.panStart) {
                var pt = utils_1.transformedPoint(_this.lastX, _this.lastY);
                var x = pt.x - _this.panStart.x;
                var y = pt.y - _this.panStart.y;
                utils_1.translate(x, y);
                if (onTranslate)
                    onTranslate(x, y);
            }
            if (onPan)
                onPan(event);
        };
        _this._onMouseUp = function (event) {
            event.preventDefault();
            return _this._onPanEnd(event);
        };
        _this._onMouseLeave = function (event) {
            event.preventDefault();
            return _this._onPanEnd(event);
        };
        _this._onTouchEnd = function (event) {
            event.preventDefault();
            if (event.targetTouches.length > 1)
                return _this._onPinchEnd(event);
            return _this._onPanEnd(event);
        };
        _this._onPanEnd = function (event) {
            var onPanEnd = _this.props.onPanEnd;
            _this.panStart = null;
            if (!_this.panned)
                _this._zoom(event.shiftKey ? -1 : 1);
            if (onPanEnd)
                onPanEnd(event);
        };
        _this._zoom = function (clicks) {
            var _a = _this.props, _b = _a.scaleFactor, scaleFactor = _b === void 0 ? 1.1 : _b, onScale = _a.onScale, onTranslate = _a.onTranslate;
            if ((_this.zoomLevel + clicks) < 0)
                return;
            _this.zoomLevel += clicks;
            var _c = utils_1.transformedPoint(_this.lastX, _this.lastY), x = _c.x, y = _c.y;
            utils_1.translate(x, y);
            if (onTranslate)
                onTranslate(x, y);
            var factor = Math.pow(scaleFactor, clicks);
            utils_1.scale(factor, factor);
            if (onScale)
                onScale(factor, factor);
            utils_1.translate(-x, -y);
            if (onTranslate)
                onTranslate(-x, -y);
        };
        _this._onPinchStart = function (event) {
            var onPinchStart = _this.props.onPinchStart;
            _this.touchCache = [];
            for (var i = 0; i < event.targetTouches.length; i += 1) {
                var touch = event.targetTouches[i];
                _this.touchCache.push(touch);
            }
            _this.lastDistance = 0;
            if (onPinchStart)
                onPinchStart(event);
        };
        _this._onPinch = function (event) {
            var _a = _this.props, onPinch = _a.onPinch, _b = _a.touchSensitivity, touchSensitivity = _b === void 0 ? constants_1.TOUCH_SENSITIVITY : _b;
            var touch1 = event.targetTouches[0];
            var touch2 = event.targetTouches[1];
            var startTouch1 = _this.touchCache.find(function (touch) { return touch.identifier === touch1.identifier; });
            var startTouch2 = _this.touchCache.find(function (touch) { return touch.identifier === touch2.identifier; });
            _this.lastX = ((touch1.clientX + touch2.clientX) / 2) - _this.node.offsetLeft;
            _this.lastY = ((touch1.clientY + touch2.clientY) / 2) - _this.node.offsetTop;
            if (startTouch1 && startTouch2) {
                var distance = utils_1.getDistance(touch1, touch2);
                if (_this.lastDistance) {
                    var initialDistance = utils_1.getDistance(startTouch1, startTouch2);
                    var ratio = (distance - _this.lastDistance) / initialDistance;
                    _this._zoom(ratio * touchSensitivity);
                }
                _this.lastDistance = distance;
            }
            if (onPinch)
                onPinch(event);
        };
        _this._onPinchEnd = function (event) {
            var onPinchEnd = _this.props.onPinchEnd;
            if (onPinchEnd)
                onPinchEnd(event);
        };
        _this._onWheel = function (event) {
            var _a = _this.props.wheelMax, wheelMax = _a === void 0 ? constants_1.WHEEL_MAX : _a;
            event.preventDefault();
            var delta = Math.round(event.deltaY / 10);
            if (delta < 0 || delta > 0) {
                if (delta > wheelMax)
                    delta = wheelMax;
                else if (delta < -wheelMax)
                    delta = -wheelMax;
                _this._zoom(delta);
            }
        };
        return _this;
    }
    Touch.prototype.componentDidMount = function () {
        window.addEventListener('resize', this._reset);
    };
    Touch.prototype.componentWillUnmount = function () {
        window.removeEventListener('resize', this._reset);
    };
    Touch.prototype.render = function () {
        var _a = this.props, onPanStart = _a.onPanStart, onPan = _a.onPan, onPanEnd = _a.onPanEnd, onTranslate = _a.onTranslate, onScale = _a.onScale, props = __rest(_a, ["onPanStart", "onPan", "onPanEnd", "onTranslate", "onScale"]);
        return (React.createElement("div", __assign({ onMouseDown: this._onMouseDown, onMouseMove: this._onMouseMove, onMouseUp: this._onMouseUp, onMouseLeave: this._onMouseLeave, onWheel: this._onWheel, onTouchStart: this._onTouchStart, onTouchMove: this._onTouchMove, onTouchEnd: this._onTouchEnd, ref: this._setup }, props)));
    };
    return Touch;
}(React.Component));
exports.Touch = Touch;
//# sourceMappingURL=Touch.js.map